@@ -1,13 +1,14 @@
 package Data::Sah::CoerceJS;
 
-our $DATE = '2016-05-19'; # DATE
-our $VERSION = '0.006'; # VERSION
+our $DATE = '2016-05-20'; # DATE
+our $VERSION = '0.007'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
 use warnings;
 use Log::Any::IfLOG '$log';
 
+use Data::Sah::CoerceCommon;
 use Nodejs::Util qw(get_nodejs_path);
 
 use Exporter qw(import);
@@ -29,42 +30,13 @@ sub _list_rule_modules {
     $mods;
 }
 
-$SPEC{gen_coercer} = {
-    v => 1.1,
-    summary => 'Generate Perl code to execute JavaScript coercer',
-    description => <<'_',
-
-This is mostly for testing. Normally the coercion rules will be used from
-`Data::Sah`.
-
-_
-    args => {
-        type => {
-            schema => 'str*', # XXX sah::typename
-            req => 1,
-            pos => 0,
-        },
-        coerce_to => {
-            schema => 'str*',
-        },
-        coerce_from => {
-            schema => ['array*', of=>'str*'],
-        },
-        dont_coerce_from => {
-            schema => ['array*', of=>'str*'],
-        },
-        source => {
-            summary => 'If set to true, will return coercer source code string'.
-                ' instead of compiled code',
-            schema => 'bool',
-        },
-    },
-    result_naked => 1,
-};
+$SPEC{gen_coercer} = $Data::Sah::CoerceCommon::gen_coercer_meta;
 sub gen_coercer {
     my %args = @_;
 
     my $type = $args{type};
+    my $rt = $args{return_type} // 'val';
+    my $rt_bv = $rt eq 'bool+val';
 
     my $all_mods = _list_rule_modules();
 
@@ -115,21 +87,36 @@ sub gen_coercer {
         for my $i (reverse 0..$#res) {
             my $res = $res[$i];
             if ($i == $#res) {
-                $expr = "($res->{expr_match}) ? ($res->{expr_coerce}) : data";
+                if ($rt_bv) {
+                    $expr = "($res->{expr_match}) ? [true, $res->{expr_coerce}] : [false, data]";
+                } else {
+                    $expr = "($res->{expr_match}) ? ($res->{expr_coerce}) : data";
+                }
             } else {
-                $expr = "($res->{expr_match}) ? ($res->{expr_coerce}) : ($expr)";
+                if ($rt_bv) {
+                    $expr = "($res->{expr_match}) ? [true, $res->{expr_coerce}] : ($expr)";
+                } else {
+                    $expr = "($res->{expr_match}) ? ($res->{expr_coerce}) : ($expr)";
+                }
             }
         }
 
         $code = join(
             "",
             "function (data) {\n",
-            "    if (data === undefined || data === null) return null;\n",
+            ($rt_bv ?
+                 "    if (data === undefined || data === null) return [false, null];\n" :
+                 "    if (data === undefined || data === null) return null;\n"
+             ),
             "    return ($expr);\n",
             "}",
         );
     } else {
-        $code = 'function (data) { return data }';
+        if ($rt_bv) {
+            $code = 'function (data) { return [false, data] }';
+        } else {
+            $code = 'function (data) { return data }';
+        }
     }
 
     if ($Log_Coercer_Code) {
@@ -166,7 +153,7 @@ sub gen_coercer {
 }
 
 1;
-# ABSTRACT: Generate Perl code to execute JavaScript coercer
+# ABSTRACT: Generate coercer code
 
 __END__
 
@@ -176,11 +163,11 @@ __END__
 
 =head1 NAME
 
-Data::Sah::CoerceJS - Generate Perl code to execute JavaScript coercer
+Data::Sah::CoerceJS - Generate coercer code
 
 =head1 VERSION
 
-This document describes version 0.006 of Data::Sah::CoerceJS (from Perl distribution Data-Sah-Coerce), released on 2016-05-19.
+This document describes version 0.007 of Data::Sah::CoerceJS (from Perl distribution Data-Sah-Coerce), released on 2016-05-20.
 
 =head1 SYNOPSIS
 
@@ -209,7 +196,7 @@ something like:
 
 =head2 gen_coercer(%args) -> any
 
-Generate Perl code to execute JavaScript coercer.
+Generate coercer code.
 
 This is mostly for testing. Normally the coercion rules will be used from
 C<Data::Sah>.
@@ -226,6 +213,12 @@ Arguments ('*' denotes required arguments):
 
 =item * B<dont_coerce_from> => I<array[str]>
 
+=item * B<return_type> => I<str> (default: "val")
+
+C<val> returns the value (possibly) coerced. C<bool+val> returns a 2-element array
+where the first element is a bool value of whether the value has been coerced,
+and the second element is the (possibly) coerced value.
+
 =item * B<source> => I<bool>
 
 If set to true, will return coercer source code string instead of compiled code.
